%Sletter alt fra minnet og laster inn lydhastigheter, 
%saltinnhold og temperatur.
clear all
clc
load towed_ctd
endelig;
salt;
temp;
close all
% Initsialisering av Lybin
lb=actxserver('LybinCom.LybinModelComBin');

% Initialiserer inputt-metoder
env = lb.invoke('IEnvironment'); % Milj¯ (bunnprofile, bunntype, vindhastighet etc)
mod = lb.invoke('IModelData'); % Modellparametre (antall strÂler, cellest¯rrelser etc)
sensor = lb.invoke('ISensor'); % Sonarparametre (strÂlebredde, kildestyrke etc)
pulse = lb.invoke('IPuls'); % pulsegenskaper (bÂndbredde, pulslengde)
platform = lb.invoke('IPlatform'); % Platformegenskaper (egenst¯y)
ocean = lb.invoke('IOcean'); % Havegenskaper (Ph, st¯ynivÂ)

% Setter inn sensordata
sensor.Depth = 40;
sensor.TiltReceiver = 0;
sensor.TiltTransmitter = 0;
sensor.SideLobeReceiver = 43;
sensor.SideLobeTransmitter = 43;
sensor.DetectionThreshold = 13;
sensor.Frequency = 1000;
sensor.DirectivityIndex = 25;
sensor.SourceLevel = 210;
sensor.BeamWidthReceiver = 10; % Vertikal strÂlebredde
sensor.BeamWidthTransmitter = 10; % Vertikal strÂlebredde

% Pulsegenskaper
pulse.Length = 1000; % i millisekund
pulse.FMBandWidth = 1000; % i Hz

% Modellegenskaper
R = 10000;
Z = 135;
R_cells = 100;
Z_cells = 34;
mod.SetRangeScaleAndRangeCells(R, R_cells); % F¯rste inputt er avstand og andre er antall rekkeviddeceller. De mÂ settes sammen og ratioen mÂ vÊre et heltall.
mod.SetRangeScaleAndRangeSteps(R, R_cells*10); % rekkevidde og steglengde. La steglengden typisk vÊre en tidel av cellest¯rrelsen
mod.TRLRays = 1000; % Antall strÂler. Bruk typisk mer enn 10 ganger antall rekkeviddeceller
mod.SetDepthScaleAndDepthCells(Z, Z_cells);% F¯rste inputt er dyp og andre er antall dybdeceller. De mÂ settes sammen og ratioen mÂ vÊre et heltall.
mod.SetDepthScaleAndDepthSteps(Z,Z_cells*10); % dybde og steglengde. La steglengden typisk vÊre en tidel av cellest¯rrelsen

% MÂlstyrke
ocean.TargetStrength = 10; % MÂlstyrken

% Laster inn lydhastighetstabell
%load sound.mat;
runs=100;
num_profiles=runs;
M=cinterpr;
S=sinterpr;
T=tinterpr;
n=size(M,1); % depth
m=size(M,2); % range

% Glatter profilen 50 ganger med Kagawas metode
%glatt_M=smooth(M,50);
%glatt_T=smooth(T,50);
%glatt_S=smooth(S,50);
%M=glatt_M;T=glatt_T; S=glatt_S;

start=1;

%stop=2000;
stop=size(M,2);

error10=zeros(1,runs-10);
norm_error10=zeros(1,runs-10);
error1=zeros(1,runs);
norm_error1=zeros(1,runs);
trl=zeros(Z_cells,R_cells,runs);
range = round(linspace(start,stop,num_profiles+1)); %legger til en grense. 
                                                        %25 blokker vil 
                                                        %f.eks kreve 26 grenser
% Dybdevektor. Trenger ikke ha flere elmenter enn antall dybdeceller
depth = round(linspace(1,135,34));

% Lager k antall hastihetsprofiler (for hver blokk)
spdprfl=zeros(length(depth),4,num_profiles);
profilesused(1).el = 1;
profilesused(2).el = [1,num_profiles];

for j=3:num_profiles
    diffel = profilesused(j-1).el(2:end) - profilesused(j-1).el(1:(end-1));
    maxint = min(find(diffel==max(diffel)));
    newprofel = ceil((profilesused(j-1).el(maxint+1) + profilesused(j-1).el(maxint))/2);
    profilesused(j).el = sort([profilesused(j-1).el,newprofel]); 
end

for i=1:num_profiles
    spdprfl(:,1,i)=depth;
    spdprfl(:,2,i)=T(depth,range(i));
    spdprfl(:,3,i)=S(depth,range(i));
    spdprfl(:,4,i)=M(depth,range(i));
end

for k=1:runs; 
    start=1;
    runs=k;
    stop=size(M,2);
    % Avstandsvektor med grensene for hver blokk

    % Skalerer opp til virkelig avstand og legger utvalget fra 1 - 30km km
    new_range=10*(range)-9;

    % Milj¯parametre
    env.WindSpeedMeasurment = [0,10000,0];
    
    % Legger hastighetsprofilene inn i modellen
    for f=profilesused(k).el
        if f==1
            SetFirstSoundSpeedProfile(env,new_range(1), new_range(2),spdprfl(:,:,1))
        else    
            AddSoundSpeedProfile(env, new_range(f), new_range(f+1),spdprfl(:,:,f))
        end
    end
    env.BottomType = [0,5000,1;5000,10000,1]; 
    prof = [0,170;10000,170;];
    env.bottomProfile = prof; 
    env.AddBottomLossFan(0,0,[0,0;10,0; 90, 0]); 
    mod.UseMeasuredBottomLoss = 1; 

    % Kj¯rer LYBIN
    lb.DoCalculation

    % Henter data
    trl(:,:,k) = 10*log10(lb.GetResultsBin(0));
   
    
    %Beregner differansevektor
    if k==1
        error1(1,k)=0;
    else
        diff1=abs(trl(:,:,k)-trl(:,:,k-1));
        diff1_sum=sum(sum(diff1));
        error1(1,k)=diff1_sum/(R_cells*Z_cells);
    end
    k   
end

figure
plot(error1)
xlabel('Antall profiler')
ylabel('Gjennomsnittlig forksjell i transmisjonstap for hver celle')
title(['Utvikling av transmisjonstap Datasett: ',num2str(1)])
savefig('konvergensraa_sammensetning1','pdf')


